import Acquisition
import Globals
import OFS.SimpleItem
import AccessControl
import OFS.PropertyManager
import OFS.ObjectManager
import webdav.Collection
import Products.XMLDocument
from Globals import HTMLFile
from DateTime import DateTime
import urllib
import string, types
from os.path import join

Views_Dir=join(INSTANCE_HOME,'lib','python', 'Products', 'NewsSyndicate', 'Views')
add_dcChannelForm=HTMLFile(join('Views', 'channelAdd'),globals())


def add_dcChannel(self, id, url=None, dc_info=None, REQUEST=None):
	"""
	Add a Channel object to a folder.
	"""
	#c=dcChannel(id)
	c=dcChannel(id, url, dc_info)
	self._setObject(id, c)
	if REQUEST is not None:
		return self.manage_main(self, REQUEST)


class dcChannel(OFS.ObjectManager.ObjectManager,
				OFS.SimpleItem.Item,
				webdav.Collection.Collection,
				Globals.Persistent,
				Acquisition.Implicit,
				AccessControl.Role.RoleManager,
				OFS.PropertyManager.PropertyManager):
	"""
	Represents a syndicated channel as defined by RSS 0.91
	
	See http://my.netscape.com/publish/help/mnn20/quickstart.html
	"""
	
	_properties=(
		{'id':'url', 'type':'string', 'mode':'w'},
		{'id':'title', 'type':'string', 'mode':'w'},
		{'id':'link', 'type':'string', 'mode':'w'},
		{'id':'description', 'type':'text', 'mode':'w'},
		{'id':'dc_title', 'type':'string', 'mode':'w'},
		{'id':'dc_subject', 'type':'tokens', 'mode':'w'},
		{'id':'dc_description', 'type':'text', 'mode':'w'},
		{'id':'dc_language', 'type':'text', 'mode':'w'},
		{'id':'ocs_updateFrequency', 'type':'int', 'mode':'w'},
		{'id':'ocs_updatePeriod', 'type':'string', 'mode':'w'},
		{'id':'fs_update', 'type':'string', 'mode':'w'},
		{'id':'ocs_format', 'type':'string', 'mode':'w'}		 
	)

	manage_options=(
		{'label':'Properties', 'action':'manage_propertiesForm'},
		{'label':'Status', 'action':'statusForm'},
		{'label':'View', 'action':'index_html'},
		{'label':'Security', 'action':'manage_access'},
	)

	__ac_permissions__=(
		('View', ('asRSS','asHTML', '', 'index_html')),
		('View management screens', ('manage_propertiesForm', 'statusForm')),
		('Change RSS Channels', ('manage_properties', 'update'))
	)

	meta_type='dcRSS Channel'

	# Don't let channel get older than a day old
	update_max_age=24*60*60
	
	# last update time
	update_last=0
	
	# maximum number of items to show
	max_display=15
	
	def __init__(self, id, url=None, dc_info=None):

		if dc_info is None:
			if url is None:
				raise 'URL not Given'
			else:
				self.url = url
				dc_info = {}
		
		info = dc_info.keys()
		
		if 'url' in info:
			self.url = dc_info['url']
		if 'dc:title' in info:
			self.dc_title = dc_info['dc:title']
		else:
			self.dc_title = ''
		if 'dc:subject' in info:
			if type(dc_info['dc:subject']) == types.ListType:
				self.dc_subject = dc_info['dc:subject']
			else:
				self.dc_subject = string.split(dc_info['dc:subject'], ',')
		else:
			self.dc_subject = ''
		if 'dc:description' in info:
			self.dc_description = dc_info['dc:description']
		else:
			self.dc_description = ''
		if 'dc:language' in info:
			self.dc_language = dc_info['dc:language']
		else:
			self.dc_language = ''
		if 'ocs:updateFrequency' in info:
			self.ocs_updateFrequency = dc_info['ocs:updateFrequency']
		else:
			self.ocs_updateFrequency=0
			
		if 'ocs:updatePeriod' in info:
			self.ocs_updatePeriod = dc_info['ocs:updatePeriod']
		else:
			self.ocs_updatePeriod = ''
		
		self.fs_update = ''
		self.id=id

		self.update()

	def stale(self):
		"""
		Is the content of the channel stale? acquires setting from NewsCenter
		"""				
		if DateTime().timeTime() - self.update_last.timeTime() > (self.default_channel_update_hrs*60*60):
			return 1


	def catalog_unindex(self):
		try:
			self.news_catalog.uncatalog_object(self.absolute_url())
		except:
			print "Foo"
			pass
		for item in self.objectValues('dcRSS Channel Item'):
			if 1:
			#try:			
			        self.news_catalog.uncatalog_object(item.absolute_url())
			#except:
			#	pass
		return 1
	
	def catalog_index(self):
		''' insert self into catalog '''
		try:
			self.news_catalog.catalog_object(self, self.absolute_url())
		except:
			pass
		for item in self.objectValues('dcRSS Channel Item'):
			if 1:
			#try:
				self.news_catalog.catalog_object(item, item.absolute_url())
			#except:
			#	pass
		return 1



	def dump_dc_info(self):
		''' returns a dict of dublin core info'''
		
		return {'url':self.url, 'dc:title':self.dc_title, 'dc:subject':self.dc_subject,
			'dc:description':self.dc_description, 'dc:language':self.dc_language,
			'ocs:updateFrequency':self.ocs_updateFrequency, 'ocs:updatePeriod':self.ocs_updatePeriod}
		
	def update(self, REQUEST=None):
		"""
		Fetch the content from a remote url and rebuild
		the channel.
		"""
		try:
			# retrieve RSS file
			f=urllib.urlopen(self.url)
		
			# parse RSS file to DOM using XML Document
			d=Products.XMLDocument.XMLDocument.Document()
			d.parse(f)
		except:
			raise "Update Error"

		# clear out the old news contents
		self.catalog_unindex()
		
		# get channel attributes
		c=d.getElementsByTagName('channel')[0]
		self.title=c.text_content('title')
		self.link=c.text_content('link')
		self.description=c.text_content('description')
		
		# get channel items
		items=d.getElementsByTagName('item')
		for item in items:
			# get information about the item
			title=item.text_content('title')
			link=item.text_content('link')
			description=item.text_content('description')
			
			# add the item
			self.add_dcItem(title, link, description)
		
		self.update_last=DateTime()

		# update the catalog
		self.catalog_index()
		
		if REQUEST is not None:
			return self.statusForm(self, REQUEST)
		
	def add_dcItem(self, title, link, description):
		"""
		Adds an item to the channel. Takes care of avoiding duplicate
		items. Returns the added item's id or None if the item would
		have been a duplicate.
		"""
		
		# clean up item attributes
		title=string.strip(title)
		link=string.strip(link)
		description=string.strip(description)
		
		# check for duplicates
		# note: this could be much more effecient
		for item in self.objectValues('dcRSS Channel Item'):
			if item.title==title and item.link==link and item.description==description:
				# item is a duplicate
				return
		
		# get a unique id
		id="i%d" % len(self.objectIds()) 
		
		# create and add the item
		item=dcItem(id, title, link, description)
		return self._setObject(id, item)
	
	statusForm=HTMLFile(join('Views','channelStatus'), globals())
	asRSS=HTMLFile(join(Views_Dir,'channelRSS'), globals())
	
	asHTML=HTMLFile('channelHTML', globals())
	index_html=HTMLFile('channelView', globals())
	slashdot_view=HTMLFile('slashdotView', globals())
	#headline_view=HTMLFile('headlineView', globals())
	
	# Indexing methods
	# ----------------
	
	def SearchableText(self):
		"""
		A concatination of searchable text.
		"""
		return "%s %s" % (self.title, self.description)
		
		
class dcItem(OFS.SimpleItem.Item,
			Globals.Persistent,
			Acquisition.Implicit,
			AccessControl.Role.RoleManager,
			OFS.PropertyManager.PropertyManager):
	"""
	Represents an item in a syndicated channel.
	"""
	
	meta_type='dcRSS Channel Item'
	
	_properties=(
		{'id':'title', 'type': 'string', 'mode': 'w'},
		{'id':'link', 'type': 'string', 'mode': 'w'},
		{'id':'description', 'type': 'text', 'mode': 'w'},
	)
	manage_options=(
		{'label':'Properties', 'action':'manage_propertiesForm'},
		{'label':'View', 'action':'index_html'},
		{'label':'Security', 'action':'manage_access'},
	)

	__ac_permissions__=(
		('View', ('', 'index_html')),
		('View management screens', ('manage_propertiesForm',)),
		('Change RSS Channels', ('manage_properties',))
	)
	
	def __init__(self, id, title="", link="", description=""):
		self.id=id
		self.title=title
		self.link=link
		self.description=description
		
	index_html=HTMLFile(join(Views_Dir,'itemView'), globals())

	
	# Indexing methods
	# ----------------
	
	def SearchableText(self):
		"""
		A concatination of searchable text.
		"""
		return "%s %s" % (self.title, self.description)
	
	def Channel(self):
		"""
		Containing channel name.
		"""
		return self.aq_parent.title
	
	def Date(self):
		"""
		Date of the item.
		"""
		return self.bobobase_modification_time()
	

		
		
